-- file/buffer/etc picker
require('telescope').setup({
  defaults = {
    mappings = {
      i = {
        ['jj'] = 'close',
      },
    },
    layout_config = {
      prompt_position = 'top',
    },
    sorting_strategy = 'ascending',
    -- use filename as preview window title
    dynamic_preview_title = true,
  },
  pickers = {
    buffers = {
      sort_lastused = true,
      sort_mru = true,
      mappings = {
        i = {
          ['<C-k>'] = 'delete_buffer'
        },
      },
    },
    find_files = {
      find_command = { "fd", "--type", "f", "--strip-cwd-prefix" }
    },
  },
})

-- use native sorter for better performance
require('telescope').load_extension('fzf')

-- custom picker to fallback to files if no git
local telescope_builtin = require('telescope.builtin')
_G.project_files = function()
  local ok = pcall(telescope_builtin.git_files, { show_untracked = true })
  if not ok then telescope_builtin.find_files({}) end
end

-- shows added/removed/changed lines
require('gitsigns').setup()

require('mini.statusline').setup({
  content = {
    -- copy-pasted from default, we just want to remove the icon
    active = function()
      local mode, mode_hl = MiniStatusline.section_mode({ trunc_width = 120 })
      local git           = MiniStatusline.section_git({ trunc_width = 75, icon = '' })
      local diagnostics   = MiniStatusline.section_diagnostics({ trunc_width = 75, icon = '' })
      local filename      = MiniStatusline.section_filename({ trunc_width = 140 })
      local fileinfo      = MiniStatusline.section_fileinfo({ trunc_width = 120 })
      local location      = MiniStatusline.section_location({ trunc_width = 75 })

      return MiniStatusline.combine_groups({
        { hl = mode_hl,                  strings = { mode } },
        { hl = 'MiniStatuslineDevinfo',  strings = { git, diagnostics } },
        '%<', -- Mark general truncate point
        { hl = 'MiniStatuslineFilename', strings = { filename } },
        '%=', -- End left alignment
        { hl = 'MiniStatuslineFileinfo', strings = { fileinfo } },
        { hl = mode_hl,                  strings = { location } },
      })
    end
  },
})

-- delete buffer while preserving layout
require('mini.bufremove').setup()

-- shows a line indicating the current indentation scope
require('mini.indentscope').setup()

-- comment actions
require('mini.comment').setup()

-- LSP completion and function signature display
require('mini.completion').setup({
  delay = {
    -- disable autocomplete
    completion = 100000000,
    info = 100,
    signature = 50,
  },
  lsp_completion = {
    source_func = 'omnifunc',
    auto_setup = false,
  },
});

-- Use Treesitter for syntax highlighting
require('nvim-treesitter.configs').setup({
  highlight = {
    enable = true,
  },
  textobjects = {
    select = {
      enable = true,
      keymaps = {
        ['i,'] = '@parameter.inner',
        ['a,'] = '@parameter.outer',
      },
    },
    swap = {
      enable = true,
      swap_next = {
        ['>,'] = '@parameter.inner',
      },
      swap_previous = {
        ['<,'] = '@parameter.inner',
      },
    },
  },
})

-- Treesitter-aware split/join
require('treesj').setup({
  use_default_keymaps = false,
})
